<?php
/**
Plugin Name: PW WooCommerce BOGO Free
Plugin URI: https://pimwick.com/pw-bogo/
Description: PW WooCommerce BOGO Free makes Buy One, Get One promotions so easy!
Version: 2.2
Author: Pimwick, LLC
Author URI: https://pimwick.com/pw-bogo/
*/

/*
Copyright (C) 2017 Pimwick, LLC

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) {
    exit;
}

// Set this to false to go back to the old style of discounts (negative fees).
const PW_BOGO_USE_COUPONS = true;

// Verify this isn't called directly.
if ( !function_exists( 'add_action' ) ) {
    echo 'Hi there!  I\'m just a plugin, not much I can do when called directly.';
    exit;
}

const PW_BOGO_COUPON_PREFIX = 'pw-bogo-coupon-';

if ( ! class_exists( 'PW_BOGO' ) ) :

register_activation_hook( __FILE__, array( 'PW_BOGO', 'plugin_activate' ) );

final class PW_BOGO {

    function __construct() {

        add_action( 'init', array( 'PW_BOGO', 'register_post_types' ), 9 );

        if ( is_admin() ) {
            add_action( 'admin_menu', array( $this, 'admin_menu' ) );
            add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
            add_filter( 'manage_edit-pw_bogo_columns', array( $this, 'edit_pw_bogo_columns' ) );
            add_action( 'add_meta_boxes_pw_bogo', array( $this, 'meta_boxes' ) );
            add_action( 'pre_get_posts', array( $this, 'pre_get_posts') );
            add_filter( 'wp_count_posts', array( $this, 'wp_count_posts' ), 10, 3 );

        } else {
            if ( true === PW_BOGO_USE_COUPONS ) {
                add_filter( 'woocommerce_cart_totals_coupon_label', array( $this, 'woocommerce_cart_totals_coupon_label' ), 10, 2 );
                add_action( 'woocommerce_removed_coupon', array( $this, 'woocommerce_removed_coupon' ) );
                add_filter( 'woocommerce_cart_totals_coupon_html', array( $this, 'woocommerce_cart_totals_coupon_html' ), 10, 2 );
            } else {
                add_action( 'woocommerce_cart_calculate_fees' , array( $this, 'woocommerce_cart_calculate_fees' ) );
                add_action( 'woocommerce_cart_contents_total' , array( $this, 'woocommerce_cart_contents_total' ) );
            }

            add_action( 'woocommerce_after_calculate_totals', array( $this, 'woocommerce_after_calculate_totals' ) );
        }
    }

    public static function plugin_activate() {
        if ( ! current_user_can( 'activate_plugins' ) )
            return;

        PW_BOGO::register_post_types();
    }

    public static function register_post_types() {
        if ( post_type_exists('pw_bogo') ) {
            return;
        }

        $labels = array(
            'name'                  => _x( 'PW BOGO', 'Post Type General Name', 'pimwick' ),
            'singular_name'         => _x( 'PW BOGO', 'Post Type Singular Name', 'pimwick' ),
            'menu_name'             => __( 'PW BOGO', 'pimwick' ),
            'name_admin_bar'        => __( 'PW BOGO', 'pimwick' ),
            'archives'              => __( 'PW BOGO Archives', 'pimwick' ),
            'parent_item_colon'     => __( 'Parent PW BOGO:', 'pimwick' ),
            'all_items'             => __( 'PW BOGO', 'pimwick' ),
            'add_new_item'          => __( 'Add New PW BOGO', 'pimwick' ),
            'add_new'               => __( 'Create New PW BOGO', 'pimwick' ),
            'new_item'              => __( 'New PW BOGO', 'pimwick' ),
            'edit_item'             => __( 'Edit PW BOGO', 'pimwick' ),
            'update_item'           => __( 'Update PW BOGO', 'pimwick' ),
            'view_item'             => __( 'View PW BOGO', 'pimwick' ),
            'search_items'          => __( 'Search PW BOGO', 'pimwick' ),
            'not_found'             => __( 'Not found', 'pimwick' ),
            'not_found_in_trash'    => __( 'Not found in Trash', 'pimwick' ),
            'featured_image'        => __( 'PW BOGO Logo', 'pimwick' ),
            'set_featured_image'    => __( 'Set PW BOGO Logo', 'pimwick' ),
            'remove_featured_image' => __( 'Remove PW BOGO Logo', 'pimwick' ),
            'use_featured_image'    => __( 'Use as PW BOGO Logo', 'pimwick' ),
            'insert_into_item'      => __( 'Insert into item', 'pimwick' ),
            'uploaded_to_this_item' => __( 'Uploaded to this item', 'pimwick' ),
            'items_list'            => __( 'PW BOGO list', 'pimwick' ),
            'items_list_navigation' => __( 'PW BOGO list navigation', 'pimwick' ),
            'filter_items_list'     => __( 'Filter PW BOGO list', 'pimwick' ),
        );

        $args = array(
            'label'                 => __( 'PW BOGO', 'pimwick' ),
            'description'           => __( 'PW BOGO', 'pimwick' ),
            'labels'                => $labels,
            'supports'              => array( 'title' ),
            'show_ui'               => true,
            'show_in_menu'          => current_user_can( 'manage_woocommerce' ) ? 'pimwick' : false,
            'has_archive'           => true
        );

        register_post_type( 'pw_bogo', $args );
    }

    public static function version() {
        $data = get_plugin_data( __FILE__ );
        return $data['Version'];
    }

    public static function wc_min_version( $version ) {
        global $woocommerce;
        return version_compare( $woocommerce->version, $version, ">=" );
    }

    function meta_boxes( $post ) {
        require( 'ui/meta-boxes.php' );

        add_meta_box( 'pw-bogo-about', __( 'About', 'pimwick' ), 'PW_BOGO_Meta_Boxes::about', 'pw_bogo', 'side', 'default' );
        add_meta_box( 'pw-bogo-description', __( 'Yes, it really is this easy!', 'pimwick' ), 'PW_BOGO_Meta_Boxes::description', 'pw_bogo', 'normal', 'default' );
        add_meta_box( 'pw-bogo-pro', __( 'Want more control?', 'pimwick' ), 'PW_BOGO_Meta_Boxes::pro', 'pw_bogo', 'normal', 'default' );
    }

    function admin_menu() {
        if ( empty ( $GLOBALS['admin_page_hooks']['pimwick'] ) ) {
            add_menu_page(
                __( 'PW BOGO', 'pimwick' ),
                __( 'Pimwick Plugins', 'pimwick' ),
                'manage_woocommerce',
                'pimwick',
                '',
                plugins_url( '/assets/images/pimwick-icon-120x120.png', __FILE__ ),
                6
            );

            add_submenu_page(
                'pimwick',
                __( 'PW BOGO', 'pimwick' ),
                __( 'Pimwick Plugins', 'pimwick' ),
                'manage_woocommerce',
                'pimwick',
                ''
            );

            remove_submenu_page('pimwick','pimwick');
        }
    }

    function admin_enqueue_scripts() {
        $data = get_plugin_data( __FILE__ );
        $version = $data['Version'];

        wp_register_style( 'pw-bogo', plugins_url( '/assets/css/style.css', __FILE__ ), array( 'woocommerce_admin_styles' ), $version );
        wp_enqueue_style( 'pw-bogo' );
    }

    function edit_pw_bogo_columns( $gallery_columns ) {
        $new_columns['cb'] = '<input type="checkbox" />';

        $new_columns['title'] = _x( 'Name', 'pimwick' );

        return $new_columns;
    }

    function woocommerce_cart_totals_coupon_label( $label, $coupon ) {
        if ( PW_BOGO::wc_min_version( '3.0' ) ) {
            if ( $this->is_bogo_coupon( $coupon->get_code() ) ) {
                $label = $coupon->get_description();
            }
        } else {
            if ( $this->is_bogo_coupon( $coupon->code ) ) {
                $label = get_post_meta( $coupon->id, 'description', true );
            }
        }

        return $label;
    }

    function woocommerce_removed_coupon( $code ) {
        if ( $this->is_bogo_coupon( $code ) ) {
            $coupon = new WC_Coupon( $code );

            if ( PW_BOGO::wc_min_version( '3.0' ) ) {
                $coupon->delete( true );
            } else {
                wp_delete_post( $coupon->ID );
            }
        }
    }

    function woocommerce_cart_totals_coupon_html( $coupon_html, $coupon ) {
        if ( PW_BOGO::wc_min_version( '3.0' ) ) {
            if ( $this->is_bogo_coupon( $coupon->get_code() ) ) {
                $coupon_html = str_replace( '[Remove]', '', $coupon_html );
            }
        } else {
            if ( $this->is_bogo_coupon( $coupon->code ) ) {
                $coupon_html = str_replace( '[Remove]', '', $coupon_html );
            }
        }

        return $coupon_html;
    }

    function woocommerce_after_calculate_totals( ) {
        // Don't want to recurse.
        remove_action( 'woocommerce_after_calculate_totals', array( $this, 'woocommerce_after_calculate_totals' ) );

        $this->maybe_add_bogo_coupon();

        add_action( 'woocommerce_after_calculate_totals', array( $this, 'woocommerce_after_calculate_totals' ) );
    }

    function woocommerce_cart_calculate_fees( $cart ) {
        $discount = $this->get_discount();
        if ( !empty( $discount ) ) {
            $cart->add_fee( $bogo->post_title, -$discount );
        }
    }

    function maybe_add_bogo_coupon() {
        if ( false === PW_BOGO_USE_COUPONS ) {
            return;
        }

        $discount = $this->get_discount();

        if ( empty( $discount ) ) {
            // Delete any invalid coupons.
            foreach ( WC()->cart->applied_coupons as $coupon_code ) {
                if ( $this->is_bogo_coupon( $coupon_code ) ) {
                    WC()->cart->remove_coupon( $coupon_code );
                }
            }
        } else {
            // Get an existing BOGO coupon.
            foreach ( WC()->cart->applied_coupons as $coupon_code ) {
                if ( $this->is_bogo_coupon( $coupon_code ) ) {
                    $code = $coupon_code;
                }
            }

            if ( !isset( $code ) ) {
                $bogos = $this->get_active_bogo();
                $bogo_title = 'BOGO';
                if ( count( $bogos ) > 0 ) {
                    $bogo = $bogos[0];
                    $bogo_title = $bogo->post_title;

                    // Create the coupon.
                    $code = PW_BOGO_COUPON_PREFIX . $this->guidv4();

                    if ( PW_BOGO::wc_min_version( '3.0' ) ) {
                        $coupon = new WC_Coupon( $code );
                        $coupon->set_code( $code );
                        $coupon->set_amount( $discount );
                        $coupon->set_usage_limit( 1 );
                        $coupon->set_free_shipping( true );
                        $coupon->update_meta_data( '_pw_bogo_id', $bogo->ID );
                        $coupon->set_description( $bogo_title );
                        $coupon->save();
                    } else {
                        $coupon = array(
                            'post_title' => $code,
                            'post_content' => '',
                            'post_status' => 'publish',
                            'post_author' => 1,
                            'post_type'     => 'shop_coupon'
                        );

                        $new_coupon_id = wp_insert_post( $coupon );

                        update_post_meta( $new_coupon_id, 'discount_type', 'fixed_cart' );
                        update_post_meta( $new_coupon_id, 'coupon_amount', $discount );
                        update_post_meta( $new_coupon_id, 'individual_use', 'no' );
                        update_post_meta( $new_coupon_id, 'product_ids', '' );
                        update_post_meta( $new_coupon_id, 'exclude_product_ids', '' );
                        update_post_meta( $new_coupon_id, 'usage_limit', '1' );
                        update_post_meta( $new_coupon_id, 'expiry_date', '' );
                        update_post_meta( $new_coupon_id, 'apply_before_tax', 'yes' );
                        update_post_meta( $new_coupon_id, 'free_shipping', 'no' );
                        update_post_meta( $new_coupon_id, 'description', $bogo_title );
                        update_post_meta( $new_coupon_id, '_pw_bogo_id', $bogo->ID );
                    }

                    WC()->cart->add_discount( $code );
                }
            } else {
                $coupon = new WC_Coupon( $code );

                if ( PW_BOGO::wc_min_version( '3.0' ) ) {
                    $coupon->set_amount( $discount );
                    $coupon->save();
                } else {
                    update_post_meta( $coupon->id, 'coupon_amount', $discount );
                }
            }

            WC()->session->set( 'refresh_totals', true );
        }
    }

    function get_discount() {
        // If we only have 1 item no need for the hokey pokey.
        if ( WC()->cart->cart_contents_count < 2 ){
            return;
        }

        $bogos = $this->get_active_bogo();

        if ( count( $bogos ) > 0 ) {
            $bogo = $bogos[0];

            foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
                // Add a record per quantity for this product to the list of products for this BOGO.
                for ( $i = 0; $i < $cart_item['quantity']; $i++ ) {
                    if ( PW_BOGO::wc_min_version( '2.7' ) ) {
                        $product_prices[] = wc_get_price_including_tax( $cart_item['data'] );
                    } else {
                        $product_prices[] = $cart_item['data']->get_price_including_tax();
                    }
                }
            }

            // Sort products in cart from highest price to lowest price so we can apply the discount
            // to the cheaper option.
            rsort( $product_prices );

            // Loop over each product, sorted by price from highest to lowest.
            $discount = 0;
            foreach ( $product_prices as $index => $price ) {
                // We only apply to the second product.
                if ( ( $index + 1 ) % 2 == 0 ) {
                    $discount += $price;
                }
            }

            if ( !empty( $discount ) ) {
                return $discount;
            }
        }

        return 0;
    }

    function is_bogo_coupon( $code ) {
        if ( strpos( $code, PW_BOGO_COUPON_PREFIX ) === 0 ) {
            return true;
        } else {
            return false;
        }
    }

    // Source: http://php.net/manual/en/function.com-create-guid.php
    function guidv4() {
        if (function_exists('com_create_guid') === true)
            return trim(com_create_guid(), '{}');

        $data = openssl_random_pseudo_bytes(16);
        $data[6] = chr(ord($data[6]) & 0x0f | 0x40); // set version to 0100
        $data[8] = chr(ord($data[8]) & 0x3f | 0x80); // set bits 6-7 to 10
        return vsprintf('%s%s-%s-%s-%s-%s%s%s', str_split(bin2hex($data), 4));
    }

    function pre_get_posts( $query ) {
        // Prevents our internal BOGO coupons from appearing in the admin area.
        if ( $query->is_main_query() || $query->is_search() ) {
            if ( $query->query['post_type'] == 'shop_coupon' ) {
                $query->set( 'meta_query', array(
                    'relation' => 'AND',
                     array(
                        'key' => '_pw_bogo_id',
                        'compare' => 'NOT EXISTS',
                     ),
                ) );

            }
        }
    }

    function wp_count_posts( $counts, $type, $perm ) {
        global $wpdb;

        // Subtract the total number of published Store Credit coupons from the counter.
        if ( 'shop_coupon' === $type && property_exists( $counts, 'publish' ) ) {
            $store_credit_coupons = $wpdb->get_var( "SELECT COUNT(*) AS total FROM {$wpdb->posts} AS p JOIN {$wpdb->postmeta} AS m ON (m.post_id = p.ID AND m.meta_key = '_pw_bogo_id') WHERE p.post_type = 'shop_coupon' AND p.post_status = 'publish'" );
            $counts->publish -= $store_credit_coupons;
        }

        return $counts;
    }

    function woocommerce_cart_contents_total( $cart_contents_total ) {
        WC()->cart->calculate_fees();
        $fees = WC()->cart->get_fees();
        $bogos = $this->get_active_bogo();
        if ( count( $bogos ) > 0 ) {
            $fee_id = sanitize_title( $bogos[0]->post_title );
            foreach ( $fees as $fee ) {
                if ( $fee->id == $fee_id ) {
                    return wc_price( WC()->cart->cart_contents_total + $fee->amount );
                }
            }
        }

        return $cart_contents_total;
    }

    private function get_active_bogo() {
        $active_bogo = get_posts( array(
            'posts_per_page' => 1,
            'post_type' => 'pw_bogo',
            'post_status' => 'publish'
        ) );

        return $active_bogo;
    }
}

new PW_BOGO();

endif;

?>